@@ -1,3 +1,17 @@
+# Copyright 2020-     Robot Framework Foundation
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 import json
 from datetime import datetime
 from typing import List, Optional, Union
@@ -8,7 +22,7 @@ from robotlibcore import keyword  # type: ignore
 
 from ..base import LibraryComponent
 from ..generated.playwright_pb2 import Request
-from ..utils import CookieType, locals_to_params, logger
+from ..utils import CookieSameSite, CookieType, locals_to_params, logger
 
 
 class Cookie(LibraryComponent):
@@ -40,7 +54,7 @@ class Cookie(LibraryComponent):
     def _get_cookies(self):
         with self.playwright.grpc_channel() as stub:
             response = stub.GetCookies(Request().Empty())
-            return response, json.loads(response.body)
+            return response, json.loads(response.json)
 
     def _format_cookies_as_string(self, cookies: List[dict]):
         pairs = []
@@ -66,7 +80,7 @@ class Cookie(LibraryComponent):
                 dot_dict[key] = cookie[key]
         return dot_dict
 
-    @keyword(tags=["Setter", "PageContent"])
+    @keyword(tags=["Setter", "BrowserControl"])
     def add_cookie(
         self,
         name: str,
@@ -77,7 +91,7 @@ class Cookie(LibraryComponent):
         expires: Optional[str] = None,
         httpOnly: Optional[bool] = None,
         secure: Optional[bool] = None,
-        sameSite: Optional[str] = None,
+        sameSite: Optional[CookieSameSite] = None,
     ):
         """Adds a cookie to currently active browser context.
 
@@ -91,7 +105,7 @@ class Cookie(LibraryComponent):
 
         ``path`` <str> Given path for the cookie. Defaults to None. Either ``url`` or ``domain`` / ``path`` pair must be set.
 
-        ``expires`` <str> Given expiry for the cookie. Can be of date format or unix time.supports
+        ``expires`` <str> Given expiry for the cookie. Can be of date format or unix time. Supports
         the same formats as the [http://robotframework.org/robotframework/latest/libraries/DateTime.html|DateTime]
         library or an epoch timestamp.
         - example: 2027-09-28 16:21:35
@@ -100,13 +114,13 @@ class Cookie(LibraryComponent):
 
         ``secure`` <bool> Sets the secure token.
 
-        ``samesite`` <"Strict"|"Lax"|"None"> Sets the samesite mode.
+        ``samesite`` < ``Strict`` | ``Lax`` | ``None`` > Sets the samesite mode.
 
         Example:
-        | `Add Cookie` | foo | bar | http://address.com/path/to/site |                                 | # Using url argument.             |
-        | `Add Cookie` | foo | bar | domain=example.com              | path=/foo/bar                   | # Using domain and url arguments. |
-        | `Add Cookie` | foo | bar | http://address.com/path/to/site | expiry=2027-09-28 16:21:35      | # Expiry as timestamp.            |
-        | `Add Cookie` | foo | bar | http://address.com/path/to/site | expiry=1822137695               | # Expiry as epoch seconds.        |
+        | `Add Cookie`   foo   bar   http://address.com/path/to/site                                     # Using url argument.
+        | `Add Cookie`   foo   bar   domain=example.com                path=/foo/bar                     # Using domain and url arguments.
+        | `Add Cookie`   foo   bar   http://address.com/path/to/site   expiry=2027-09-28 16:21:35        # Expiry as timestamp.
+        | `Add Cookie`   foo   bar   http://address.com/path/to/site   expiry=1822137695                 # Expiry as epoch seconds.
         """
         params = locals_to_params(locals())
         if expires:
@@ -123,7 +137,7 @@ class Cookie(LibraryComponent):
         except ValueError:
             return int(convert_date(expiry, result_format="epoch"))
 
-    @keyword(tags=["Setter", "PageContent"])
+    @keyword(tags=["Setter", "BrowserControl"])
     def delete_all_cookies(self):
         """Deletes all cookies from the currently active browser context."""
         with self.playwright.grpc_channel() as stub:
@@ -146,12 +160,14 @@ class Cookie(LibraryComponent):
         )
         logger.warn("Cookie monster ate all cookies!!")
 
-    @keyword(tags=["Getter", "PageContent"])
+    @keyword(tags=["Getter", "BrowserControl"])
     def get_cookie(
         self, cookie: str, return_type: CookieType = CookieType.dictionary
     ) -> Union[DotDict, str]:
         """Returns information of cookie with ``name`` as a Robot Framework dot dictionary or a string.
 
+        ``cookie`` <str> Name of the cookie to be retrieved.
+
         If ``return_type`` is ``dictionary`` or ``dict`` then keyword returns a of Robot Framework
         [https://robotframework.org/robotframework/latest/RobotFrameworkUserGuide.html#accessing-list-and-dictionary-items|dot dictionary]
         The dictionary contains all possible key value pairs of the cookie. If ``return_type`` is ``string`` or ``str``,
@@ -161,7 +177,7 @@ class Cookie(LibraryComponent):
         If no cookie is found with ``name`` keyword fails. The cookie dictionary contains
         details about the cookie. Keys available in the dictionary are documented in the table below.
 
-        | Value    | Explanation                                                                                |
+        | *Value*  | *Explanation*                                                                              |
         | name     | The name of a cookie, mandatory.                                                           |
         | value    | Value of the cookie, mandatory.                                                            |
         | url      | Define the scope of the cookie, what URLs the cookies should be sent to.                   |
@@ -177,9 +193,9 @@ class Cookie(LibraryComponent):
         for details about each attribute.
 
         Example:
-        | ${cookie} =     | Get Cookie            | Foobar  |
-        | Should Be Equal | ${cookie.value}       | Tidii   |
-        | Should Be Equal | ${cookie.expiry.year} | ${2020} |
+        | ${cookie}=        Get Cookie              Foobar
+        | Should Be Equal   ${cookie.value}         Tidii
+        | Should Be Equal   ${cookie.expiry.year}   ${2020}
         """
         _, cookies = self._get_cookies()
         for cookie_dict in cookies:
