@@ -1,6 +1,20 @@
+# Copyright 2020-     Robot Framework Foundation
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 import json
 from pathlib import Path
-from typing import Dict, Optional
+from typing import Any, Dict, Optional
 
 from robotlibcore import keyword  # type: ignore
 
@@ -8,7 +22,9 @@ from ..base import LibraryComponent
 from ..generated.playwright_pb2 import Request
 from ..utils import logger
 from ..utils.data_types import (
-    AlertAction,
+    BoundingBox,
+    Coordinates,
+    DialogAction,
     KeyAction,
     KeyboardInputAction,
     KeyboardModifier,
@@ -136,10 +152,10 @@ class Interaction(LibraryComponent):
         supported input keys.
         [https://github.com/microsoft/playwright/blob/master/docs/api.md#pagepressselector-key-options | Playwright docs for press.]
 
-        Example: 
+        Example:
 
-        | Keyword      | Selector                   | Keys | Keys | Keys | Keys | Keys        | Keys |
-        | Press Keys   | //*[@id="username_field"]  |  h   |   e  |  l   |  o   |  ArrowLeft  |  l   |
+        | # Keyword       Selector                    *Keys
+        | Press Keys      //*[@id="username_field"]    h    e   l   o   ArrowLeft   l
         """  # noqa
         with self.playwright.grpc_channel() as stub:
             response = stub.Press(Request().PressKeys(selector=selector, key=keys))
@@ -160,47 +176,123 @@ class Interaction(LibraryComponent):
         self,
         selector: str,
         button: MouseButton = MouseButton.left,
-        click_count: int = 1,
+        clickCount: int = 1,
         delay: Optional[str] = None,
-        position_x: Optional[int] = None,
-        position_y: Optional[int] = None,
+        position_x: Optional[float] = None,
+        position_y: Optional[float] = None,
         force: bool = False,
+        noWaitAfter: bool = False,
         *modifiers: KeyboardModifier,
     ):
-        """Simulates mouse click with multiple options on the element found by ``selector``.
+        """Moves the virtual mouse with multiple options on the element found by ``selector``.
+
+        This method hovers over an element matching ``selector`` by performing the following steps:
+        - Find an element match matching ``selector``. If there is none, wait until a matching element is attached to the DOM.
+        - Wait for actionability checks on the matched element, unless ``force`` option is set. If the element is detached during the checks, the whole action is retried.
+        - Scroll the element into view if needed.
+        - Use `Mouse Move` to hover over the center of the element, or the specified ``position``.
 
         ``selector`` <str> Selector element to click. **Required**
 
-        ``button`` <left|right|middle> Defaults to ``left`` if invalid.
+        ``button`` < ``left`` | ``middle`` |  ``right``> The button that shall be used for clicking.
 
-        ``click_count`` <int> Defaults to 1.
+        ``clickCount`` <int> How many time shall be clicked.
 
-        ``delay`` <robot time> Time to wait between mousedown and mouseup in milliseconds.
-        Defaults to 0.
+        ``delay`` <robot time str> Time to wait between mousedown and mouseup and next click.
 
-        ``position_x`` & ``position_y`` <int> A point to click relative to the
-        top-left corner of element padding box.
+        *Caution: be aware that if the delal leats to a total time that exceets the timeout, the keyword fails*
+
+        ``position_x`` & ``position_y`` <float> A point to click relative to the
+        top-left corner of element boundingbox. Only positive values within the boundingbox are allowed.
         If not specified, clicks to some visible point of the element.
 
+        *Caution: even with 0, 0 might click a few pixels off from the corner of the boundingbox. Click uses detection to find the first clickable point.*
+
         ``force`` <bool> Set to True to skip Playwright's [https://github.com/microsoft/playwright/blob/master/docs/actionability.md | Actionability checks].
 
-        ``*modifiers`` ``<list<"Alt"|"Control"|"Meta"|"Shift">>``
+        ``*modifiers`` < ``Alt`` | ``Control`` | ``Meta`` | ``Shift`` >
         Modifier keys to press. Ensures that only these modifiers are pressed
         during the click, and then restores current modifiers back.
         If not specified, currently pressed modifiers are used.
         """
         with self.playwright.grpc_channel() as stub:
-            options = {"button": button.name, "clickCount": click_count, "force": force}
+            options = {
+                "button": button.name,
+                "clickCount": clickCount,
+                "force": force,
+                "noWaitAfter": noWaitAfter,
+            }
             if delay:
                 options["delay"] = timestr_to_millisecs(delay)
-            if position_x and position_y:
+            # Without the != None 0 being falsy causes issues
+            if position_x is not None and position_y is not None:
                 positions: Dict[str, object] = {"x": position_x, "y": position_y}
                 options["position"] = positions
             if modifiers:
                 options["modifiers"] = [m.name for m in modifiers]
             options_json = json.dumps(options)
             logger.debug(f"Click Options are: {options_json}")
-            response = stub.Click(
+            response = stub.ClickWithOptions(
+                Request().ElementSelectorWithOptions(
+                    selector=selector, options=options_json
+                )
+            )
+            logger.debug(response.log)
+
+    @keyword(tags=["Setter", "PageContent"])
+    def hover(
+        self,
+        selector: str,
+        position_x: Optional[float] = None,
+        position_y: Optional[float] = None,
+        force: bool = False,
+        *modifiers: KeyboardModifier,
+    ):
+        """Simulates mouse click with multiple options on the element found by ``selector``.
+
+        This keyword clicks an element matching ``selector`` by performing the following steps:
+        - Find an element matches selector. If there is none, wait until a matching element is attached to the DOM.
+        - Wait for actionability checks on the matched element, unless ``force`` option is set. If the element is detached during the checks, the whole action is retried.
+        - Scroll the element into view if needed.
+        - Use `Mouse Button` to click in the center of the element, or the specified position.
+        - Wait for initiated navigations to either succeed or fail, unless ``noWaitAfter`` option is set.
+
+        ``selector`` <str> Selector element to click. **Required**
+
+        ``button`` < ``left`` | ``right`` | ``middle`` > Defaults to ``left`` if invalid.
+
+        ``click_count`` <int> Defaults to 1.
+
+        ``delay`` <robot time str> Time to wait between mousedown and mouseup.
+        Defaults to 0.
+
+        ``position_x`` & ``position_y`` <int> A point to click relative to the
+        top-left corner of element boundingbox. Only positive values within the boundingbox are allowed.
+        If not specified, clicks to some visible point of the element.
+
+        ``force`` <bool> Set to True to skip Playwright's [https://github.com/microsoft/playwright/blob/master/docs/actionability.md | Actionability checks].
+
+        ``noWaitAfter`` <bool> Actions that initiate navigations are waiting for
+        these navigations to happen and for pages to start loading.
+        You can opt out of waiting via setting this flag.
+        You would only need this option in the exceptional cases such as navigating
+        to inaccessible pages. Defaults to ``False``.
+
+        ``*modifiers`` < ``Alt`` | ``Control`` | ``Meta`` | ``Shift`` >
+        Modifier keys to press. Ensures that only these modifiers are pressed
+        during the click, and then restores current modifiers back.
+        If not specified, currently pressed modifiers are used.
+        """
+        with self.playwright.grpc_channel() as stub:
+            options: Dict[str, Any] = {"force": force}
+            if position_x and position_y:
+                positions: Dict[str, object] = {"x": position_x, "y": position_y}
+                options["position"] = positions
+            if modifiers:
+                options["modifiers"] = [m.name for m in modifiers]
+            options_json = json.dumps(options)
+            logger.debug(f"Hover Options are: {options_json}")
+            response = stub.Hover(
                 Request().ElementSelectorWithOptions(
                     selector=selector, options=options_json
                 )
@@ -214,7 +306,7 @@ class Interaction(LibraryComponent):
         ``selector`` <str> Selector of the element. **Required**
 
         If there's no element matching selector, the method waits until a
-        matching element appears in the DOM.
+        matching element appears in the DOM. Timeouts after 10 seconds.
         """
         with self.playwright.grpc_channel() as stub:
             response = stub.Focus(Request().ElementSelector(selector=selector))
@@ -254,7 +346,7 @@ class Interaction(LibraryComponent):
 
         Matches based on the chosen attribute with list of ``values``.
         Possible attributes to match options by:
-        ``attribute`` ``<"value"|"label"|"text"|"index">``
+        ``attribute`` < ``value`` | ``label `` | ``text`` | ``index`` >
 
         If no values to select are passed will deselect options in element.
         """
@@ -309,16 +401,16 @@ class Interaction(LibraryComponent):
             if log_response:
                 logger.debug(response.log)
 
-    @keyword(tags=["Setter", "PageContent", "EventHandler"])
+    @keyword(tags=["Setter", "PageContent"])
     def upload_file(self, path: str):
-        """ Upload file from ``path`` into next file chooser dialog on page.
+        """Upload file from ``path`` into next file chooser dialog on page.
 
         ``path`` <str> Path to file to be uploaded.
 
         Example use:
 
-            | Upload File |  ${CURDIR}/test_upload_file
-            | Click       |  \\#file_chooser
+        | Upload File    ${CURDIR}/test_upload_file
+        | Click          \\#file_chooser
 
         """
         p = Path(path)
@@ -327,26 +419,26 @@ class Interaction(LibraryComponent):
             response = stub.UploadFile(Request().FilePath(path=str(p)))
             logger.debug(response.log)
 
-    @keyword(tags=["PageContent", "EventHandler"])
-    def handle_alert(self, action: AlertAction, prompt_input: str = ""):
-        """ Handle next dialog on page with ``action``. Dialog can be any of alert,
+    @keyword(tags=["PageContent"])
+    def handle_future_dialogs(self, action: DialogAction, prompt_input: str = ""):
+        """Handle next dialog on page with ``action``. Dialog can be any of alert,
         beforeunload, confirm or prompt.
 
-            ``action`` <accept|dismiss> How to handle the alert. **Required**
+            ``action`` < ``acceppt`` | ``dismiss`` > How to handle the alert. **Required**
 
             ``prompt_input`` <str> The value to enter into prompt. Only valid if
             ``action`` equals accept. Defaults to empty string.
         """
 
         with self.playwright.grpc_channel() as stub:
-            if prompt_input and action is not AlertAction.accept:
+            if prompt_input and action is not DialogAction.accept:
                 raise ValueError("prompt_input is only valid if action is 'accept'")
             response = stub.HandleAlert(
                 Request().AlertAction(alertAction=action.name, promptInput=prompt_input)
             )
             logger.debug(response.log)
 
-    @keyword(tags=["VirtualMouse", "PageContent"])
+    @keyword(tags=["Setter", "PageContent"])
     def mouse_button(
         self,
         action: MouseButtonAction,
@@ -356,28 +448,45 @@ class Interaction(LibraryComponent):
         clickCount: int = 1,
         delay: int = 0,
     ):
-        """ Click or hold a mouse button down.
+        """Click, hold a mouse button down or release it.
 
-            ``action`` <click|up|down>
+        Moving the mouse between holding down and releasing it for example is possible with `Mouse Move`.
 
-            ``x`` <int>
+        ``action`` < ``click`` | ``up`` | ``down`` > Determines if it is a mouseclick, holding down a key or releasing it.
 
-            ``y`` <int>
+        ``x`` <int> and ``y`` <int> Coordinates for a click only. Defaults to None.
+        **Required** if action is a click.
 
-            ``button`` <left|right|middle> Defaults to ``left`` if invalid.
+        ``button`` < ``left``| ``right`` | ``middle`` > Defaults to ``left`` if invalid.
 
-            ``clickCount`` <int> Deterine how often shall be clicked. Defaults to 1.
+        ``clickCount`` <int> Deterine how often shall be clicked. Defaults to 1.
 
-            ``delay`` <int> Delay can only be set if the action is click.
+        ``delay`` <int> Delay in ms between the mousedown and mouseup event. Can only be set if the action is click.
         """
         with self.playwright.grpc_channel() as stub:
             body: MouseOptionsDict = {}
             if delay and action is not MouseButtonAction.click:
-                raise ValueError("Delay is only valid on a mouse click")
-            if action is MouseButtonAction.down or action is MouseButtonAction.up:
-                if x or y:
+                raise ValueError("Delay is only valid on 'click' action.")
+            if action == MouseButtonAction.click:
+                body = {}
+                if x and y:
+                    body["x"] = float(x)
+                    body["y"] = float(y)
+                else:
                     raise ValueError(
-                        "Coordinates are not valid on MouseAction.up or MouseAction.down"
+                        f"`Mouse Button    Click` requires that x and y are set! x: {x}, y: {y}"
+                    )
+                body["options"] = {
+                    "button": button.name,
+                    "clickCount": clickCount,
+                    "delay": delay,
+                }
+            else:
+                if x and y:
+                    self.mouse_move(x, y)
+                else:
+                    logger.info(
+                        f"No coordinates where set. Action will appear at current position. x: {x}, y {y}"
                     )
                 body = {
                     "options": {
@@ -386,39 +495,112 @@ class Interaction(LibraryComponent):
                         "delay": delay,
                     }
                 }
-            else:
-                body = {}
-                if x:
-                    body["x"] = float(x)
-                if y:
-                    body["y"] = float(y)
-                body["options"] = {
-                    "button": button.name,
-                    "clickCount": clickCount,
-                    "delay": delay,
-                }
 
             response = stub.MouseButton(
                 Request().MouseButtonOptions(action=action.name, json=json.dumps(body))
             )
             logger.debug(response.log)
 
-    @keyword(tags=["VirtualMouse", "PageContent"])
+    @keyword(tags=["Setter", "PageContent"])
+    def drag_and_drop(self, selector_from: str, selector_to: str, steps: int = 1):
+        """Executes a Drag&Drop operation from the element selected by ``selector_from``
+        to the element selected by ``selector_to``.
+
+        First it moves the mouse to the start-point,
+        then presses the left mouse button,
+        then moves to the end-point in specified number of steps,
+        then releases the mouse button.
+
+        Start- and end-point are defined by the center of the elements boundingbox.
+
+        ``selector_from``: <float> identifies the element, which center is the start-point. **Required**
+
+        ``selector_to``: <float> identifies the element, which center is the end-point. **Required**
+
+        ``steps``: <int> defines how many intermediate mouse move events are sent.
+        """
+        from_bbox = self.library.get_boundingbox(selector_from)
+        from_xy = self._center_of_boundingbox(from_bbox)
+        to_bbox = self.library.get_boundingbox(selector_to)
+        to_xy = self._center_of_boundingbox(to_bbox)
+        self.mouse_button(MouseButtonAction.down, **from_xy)
+        self.mouse_move(**to_xy, steps=steps)
+        self.mouse_button(MouseButtonAction.up)
+
+    @keyword(tags=["Setter", "PageContent"])
+    def drag_and_drop_by_coordinates(
+        self, from_x: float, from_y: float, to_x: float, to_y: float, steps: int = 1
+    ):
+        """Executes a Drag&Drop operation from a coordinate to another coordinate.
+
+        First it moves the mouse to the start-point,
+        then presses the left mouse button,
+        then moves to the end-point in specified number of steps,
+        then releases the mouse button.
+
+        Start- and end-point are defined by ``x`` and ``y`` coordinates relative to
+        the top left corner of the pages viewport.
+
+        ``from_x`` & ``from_y``: <float> identify the the start-point. **Required**
+
+        ``to_x`` & ``to_y``: <float> identify the the end-point. **Required**
+
+        ``steps``: <int> defines how many intermediate mouse move events are sent.
+        """
+        self.mouse_button(MouseButtonAction.down, x=from_x, y=from_y)
+        self.mouse_move(x=to_x, y=to_y, steps=steps)
+        self.mouse_button(MouseButtonAction.up)
+
+    @staticmethod
+    def _center_of_boundingbox(boundingbox: BoundingBox) -> Coordinates:
+        center = Coordinates()
+        center["x"] = boundingbox["x"] + (boundingbox["width"] / 2)
+        center["y"] = boundingbox["y"] + (boundingbox["height"] / 2)
+        return center
+
+    @keyword(tags=["Setter", "PageContent"])
     def mouse_move(self, x: float, y: float, steps: int = 1):
-        """ Instead of selectors command mouse with coordinates.
-            The Click commands will leave the virtual mouse on the specified coordinates.
-            ``x``
-            ``y``
-            ``steps``
+        """Instead of selectors command mouse with coordinates.
+        The Click commands will leave the virtual mouse on the specified coordinates.
+
+        ``x`` <float> ``y`` <float> are absolute coordinates starting at the top left
+        of the page.
+
+        ``steps`` <int> Number of intermediate steps for the mouse event.
         """
         with self.playwright.grpc_channel() as stub:
             body: MouseOptionsDict = {"x": x, "y": y, "options": {"steps": steps}}
+            logger.info(f"Moving mouse to x: {x}, y: {y} coordinates.")
             response = stub.MouseMove(Request().Json(body=json.dumps(body)))
             logger.debug(response.log)
 
-    @keyword(tags=["VirtualKeyboard", "PageContent"])
+    @keyword(tags=["Setter", "PageContent"])
     def keyboard_key(self, action: KeyAction, key: str):
-        """ Press a keyboard key on the virtual keyboard or set a key up or down
+        """Press a keyboard key on the virtual keyboard or set a key up or down.
+
+        ``action`` < ``up`` | ``down`` | ``press`` > Determine whether the key should be released,
+        hold or pressed. ``down`` or ``up`` are useful for combinations i.e. with Shift.
+        **Required**
+
+        ``key`` <str> The key to be pressed. An example of valid keys are:
+
+        ``F1`` - ``F12``, ``Digit0`` - ``Digit9``, ``KeyA``- ``KeyZ``, ``Backquote``, ``Minus``,
+        ``Equal``, ``Backslash``, ``Backspace``, ``Tab``, ``Delete``, ``Escape``, ``ArrowDown``,
+        ``End``, ``Enter``, ``Home``, ``Insert``, ``PageDown``, ``PageUp``, ``ArrowRight``, ``ArrowUp``
+        , etc.
+
+        Useful keys for ``down`` and ``up`` for example are:
+
+        ``Shift``, ``Control``, ``Alt``, ``Meta``, ``ShiftLeft``
+
+        Example excecution:
+        | Keyboard Key    press    S
+        | Keyboard Key    down     Shift
+        | Keyboard Key    press    ArrowLeft
+        | Keyboard Key    press    Delete
+        | Keyboard Key    up       Shift
+
+        Note: Capital letters don't need to be written by the help of Shift. You can type them in directly.
         """
         with self.playwright.grpc_channel() as stub:
             response = stub.KeyboardKey(
@@ -426,16 +608,21 @@ class Interaction(LibraryComponent):
             )
             logger.debug(response.log)
 
-    @keyword(tags=["VirtualKeyboard", "PageContent"])
+    @keyword(tags=["Setter", "PageContent"])
     def keyboard_input(self, action: KeyboardInputAction, input: str, delay=0):
-        """ Input text into page with virtual keyboard, inserting with
-            ``action``
-                ``insertText`` Dispatches only input event, does not emit the keydown, keyup or keypress events.
-                ``type`` Sends a keydown, keypress/input, and keyup event for each character in the text.
+        """Input text into page with virtual keyboard.
 
-                To press a special key, like Control or ArrowDown, use keyboard.press.
+        ``action`` < ``insertText`` | ``type`` > **Required**
 
-            NOTE Modifier keys DO NOT effect these methods. For testing modifier effects use single key presses with ``Keyboard Key  press``
+            - ``insertText`` Dispatches only input event, does not emit the keydown, keyup or keypress events.
+
+            - ``type`` Sends a keydown, keypress/input, and keyup event for each character in the text.
+
+        ``input`` <str> The inputstring to be typed. No special keys possible. **Required**
+
+        Note: To press a special key, like Control or ArrowDown, use keyboard.press.
+        Modifier keys DO NOT effect these methods. For testing modifier effects use single key
+        presses with ``Keyboard Key  press``
 
         """
         with self.playwright.grpc_channel() as stub:
